use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006002
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
AIX
BENCHMARKING
CRLF
SHA
NFS
canonpath
codepoints
cwd
dirname
fatalize
lstat
mkpath
opena
openr
openrw
openw
realpath
stringifying
subclasses
touchpath
UNC
unlinked
utf
David
Golden
dagolden
Chris
Williams
bingos
Steinbrunner
dsteinbrunner
Gabor
Szabo
szabgab
Gabriel
Andrade
gabiruh
George
Hartzell
hartzell
Geraud
Continsouzas
geraud
Goro
Fuji
gfuji
Karen
Etheridge
ether
Martin
Kjeldsen
mk
Michael
Schwern
mschwern
Toby
Inkster
tobyink
김도형
Keedi
Kim
keedi
lib
Path
Tiny
